import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import ReachAlert from "@reach/alert";
import { Text } from "./Text";
import { CloseButton } from "./IconButton";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
import { IconWrapper } from "./IconWrapper";
import { IconInfo, IconCheckCircle, IconAlertCircle, IconAlertOctagon, IconHelpCircle } from "./Icons";
const alertIntentions = (theme) => ({
    info: theme.colors.intent.none,
    success: theme.colors.intent.success,
    question: theme.colors.intent.primary,
    danger: theme.colors.intent.danger,
    warning: theme.colors.intent.warning
});
const icons = {
    info: _jsx(IconInfo, {}, void 0),
    success: _jsx(IconCheckCircle, {}, void 0),
    warning: _jsx(IconAlertCircle, {}, void 0),
    danger: _jsx(IconAlertOctagon, {}, void 0),
    question: _jsx(IconHelpCircle, {}, void 0)
};
/**
 * Use an alert to inform users of important information.
 * To display an alert in a toast notification, use the toast module.
 */
export const Alert = (_a) => {
    var { children, title, id, subtitle, component, elevation = "xs", onRequestClose, intent = "info" } = _a, other = __rest(_a, ["children", "title", "id", "subtitle", "component", "elevation", "onRequestClose", "intent"]);
    const theme = useTheme();
    const intentions = React.useMemo(() => alertIntentions(theme), [theme]);
    const dark = theme.colors.mode === "dark";
    const color = intentions[intent];
    let accent = dark ? color.light : color.base;
    if (dark && intent === "info") {
        accent = color.light;
    }
    const icon = icons[intent];
    const contents = title ? (_jsxs("div", Object.assign({ className: "Alert__title", css: { display: "flex", alignItems: "flex-start" } }, { children: [_jsx("div", Object.assign({ css: { flex: "0 0 auto", marginTop: "2px" } }, { children: _jsx(IconWrapper, Object.assign({ size: "md", color: accent }, { children: icon }), void 0) }), void 0),
            _jsxs("div", Object.assign({ className: "Alert__text-content", css: { marginLeft: theme.spaces.md } }, { children: [_jsx(Text, Object.assign({ className: "Alert__title-text", id: id, css: { margin: 0 }, variant: "h6" }, { children: title }), void 0),
                    subtitle && (_jsx(Text, Object.assign({ className: "Alert__title-subtitle", muted: true, css: {
                            fontSize: theme.fontSizes[0]
                        } }, { children: subtitle }), void 0)), children] }), void 0)] }), void 0)) : (children);
    const Component = component || ReachAlert;
    return (_jsx(Component, Object.assign({ className: "Alert", css: {
            backgroundColor: dark
                ? theme.colors.background.tint1
                : theme.colors.background.default,
            overflow: "hidden",
            position: "relative",
            boxShadow: theme.shadows[elevation],
            borderRadius: theme.radii.md
        } }, other, { children: _jsxs("div", { children: [_jsx("div", { className: "Alert__bar", css: {
                        width: theme.radii.md,
                        position: "absolute",
                        top: 0,
                        left: 0,
                        bottom: 0,
                        backgroundColor: accent
                    } }, void 0),
                _jsxs("div", Object.assign({ className: "Alert__content", css: {
                        display: "flex",
                        alignItems: "flex-start",
                        padding: `${theme.spaces.md} ${theme.spaces.md}`,
                        paddingRight: onRequestClose ? "3.5rem" : undefined
                    } }, { children: [contents, onRequestClose && (_jsx(CloseButton, { css: {
                                marginTop: "-0.45rem",
                                right: theme.spaces.sm,
                                position: "absolute"
                            }, onClick: onRequestClose }, void 0))] }), void 0)] }, void 0) }), void 0));
};
Alert.displayName = "Alert";
Alert.propTypes = {
    onRequestClose: PropTypes.func,
    subtitle: PropTypes.string,
    title: PropTypes.string,
    id: PropTypes.oneOfType([PropTypes.string]),
    children: PropTypes.node,
    intent: PropTypes.oneOf([
        "info",
        "success",
        "warning",
        "danger",
        "question"
    ]),
    elevation: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
//# sourceMappingURL=Alert.js.map