import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { Button, ButtonRefComp, getHeight } from "./Button";
import VisuallyHidden from "@reach/visually-hidden";
import PropTypes from "prop-types";
import { IconWrapper } from "./IconWrapper";
import { IconX } from "./Icons";
import { useTheme } from "./Theme/Providers";
/**
 * A component which composes Button and Icon to provide
 * interactive icon elements.
 */
export const IconButton = (_a) => {
    var { label, size = "md", icon, onPress, color = "currentColor" } = _a, other = __rest(_a, ["label", "size", "icon", "onPress", "color"]);
    const theme = useTheme();
    return (_jsxs(Button, Object.assign({ size: size, css: {
            padding: 0,
            [theme.mediaQueries.sm]: {
                padding: 0,
            },
            [theme.mediaQueries.lg]: {
                padding: 0, //底层Button有设置[theme.xx.lg],这里就也得加[theme.xx.lg]，不然会被无情颠覆。优先级毛病？
            },
            width: getHeight(size)
        }, onPress: onPress }, other, { children: [_jsx(VisuallyHidden, { children: label }, void 0),
            _jsx(IconWrapper, Object.assign({ color: color, size: size }, { children: icon }), void 0)] }), void 0));
};
/**
 * 旧版本 可传递 ref
 * //无法自动推断onPress(e:)类型 Parameter 'e' implicitly has an 'any' type ; onPress:(e: any)=>{e!.};
 * 必须加上:  : RReact.FunctionComponent<IconButtonProps> =  定义体才行啊。
 */
export const IconRefButton = React.forwardRef((_a, ref) => {
    var { label, size = "md", icon, onPress, color = "currentColor" } = _a, other = __rest(_a, ["label", "size", "icon", "onPress", "color"]);
    const theme = useTheme();
    return (_jsxs(ButtonRefComp, Object.assign({ ref: ref, size: size, css: {
            padding: 0,
            [theme.mediaQueries.sm]: {
                padding: 0,
            },
            [theme.mediaQueries.lg]: {
                padding: 0, //底层Button有设置[theme.xx.lg],这里就也得加[theme.xx.lg]，不然会被无情颠覆。优先级毛病？
            },
            width: getHeight(size)
        }, onPress: onPress }, other, { children: [_jsx(VisuallyHidden, { children: label }, void 0),
            _jsx(IconWrapper, Object.assign({ color: color, size: size }, { children: icon }), void 0)] }), void 0));
});
IconButton.displayName = "IconButton";
IconButton.propTypes = {
    icon: PropTypes.node.isRequired,
    color: PropTypes.string,
    label: PropTypes.string.isRequired,
    variant: PropTypes.oneOf(["outline", "default", "ghost"]),
    size: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
IconRefButton.displayName = "IconRefButton";
IconRefButton.propTypes = {
    onPress: PropTypes.func,
    icon: PropTypes.node.isRequired,
    color: PropTypes.string,
    label: PropTypes.string.isRequired,
    variant: PropTypes.oneOf(["outline", "default", "ghost"]),
    size: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
export const CloseButton = (_a) => {
    var { label = "Close" } = _a, other = __rest(_a, ["label"]);
    return (_jsx(IconButton, Object.assign({ variant: "ghost", label: label, icon: _jsx(IconX, {}, void 0) }, other), void 0));
};
CloseButton.propTypes = {
    label: PropTypes.string,
    color: PropTypes.string,
    variant: PropTypes.oneOf(["outline", "default", "ghost"]),
    size: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"])
};
//# sourceMappingURL=IconButton.js.map