import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Layer } from "./Layer";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
//没有对外输出TableSectionContext的；无法import {TableSectionContext} from "customize-easy-ui-component";
const TableSectionContext = React.createContext({
    type: "TableHead"
});
//旧版本的Table才需要使用到的 context变量； 新版本已经改成参数适应;
const TableContext = React.createContext({ fixed: false });
//等待删除 代码
//@Deprecated
const TableOld = (_a) => {
    var { children, minWidth, fixed } = _a, other = __rest(_a, ["children", "minWidth", "fixed"]);
    const theme = useTheme();
    return (_jsx("div", Object.assign({ css: {
            width: "100%",
            display: "block",
            overflowX: minWidth ? "auto" : "initial"
        } }, { children: _jsxs("table", Object.assign({ css: {
                borderSpacing: 0,
                borderCollapse: "separate",
                width: "100%",
                fontFamily: theme.fonts.base,
                WebkitAppearance: "none",
                WebkitFontSmoothing: "antialiased",
                display: "table",
                minWidth,
                tableLayout: fixed ? "fixed" : undefined
            } }, other, { children: [fixed && (_jsx("colgroup", { children: fixed.map((width, i) => {
                        return _jsx("col", { width: width }, i);
                    }) }, void 0)),
                _jsx(TableContext.Provider, Object.assign({ value: { fixed: fixed ? true : false } }, { children: children }), void 0)] }), void 0) }), void 0));
};
/**
支持打印自适应，支持手机自适应。
 */
export const Table = (_a) => {
    var { children, minWidth, fixed, printColWidth } = _a, other = __rest(_a, ["children", "minWidth", "fixed", "printColWidth"]);
    const theme = useTheme();
    return (_jsx("div", Object.assign({ css: {
            width: "100%",
            display: "block",
            overflowX: minWidth ? "auto" : "initial"
        } }, { children: _jsxs("table", Object.assign({ css: {
                borderSpacing: 0,
                borderCollapse: "separate",
                width: "100%",
                fontFamily: theme.fonts.base,
                WebkitAppearance: "none",
                WebkitFontSmoothing: "antialiased",
                display: "table",
                minWidth,
                tableLayout: fixed ? "fixed" : undefined,
                "@media print": {
                    tableLayout: printColWidth ? "fixed" : fixed ? "fixed" : undefined,
                }
            } }, other, { children: [fixed && (_jsx("colgroup", Object.assign({ css: {
                        "@media print": {
                            display: printColWidth ? 'none' : undefined,
                        },
                    } }, { children: fixed.map((width, i) => {
                        return _jsx("col", { width: width }, i);
                    }) }), void 0)),
                printColWidth && (_jsx("colgroup", Object.assign({ css: {
                        "@media not print": {
                            display: 'none',
                        },
                        "@media print": {
                            display: undefined,
                        },
                    } }, { children: printColWidth.map((width, i) => {
                        return _jsx("col", { width: width }, i);
                    }) }), void 0)), children] }), void 0) }), void 0));
};
Table.propTypes = {
    minWidth: PropTypes.string,
    fixed: PropTypes.arrayOf(PropTypes.string),
    printColWidth: PropTypes.arrayOf(PropTypes.string),
    children: PropTypes.node
};
export function TableHead(_a) {
    var { children } = _a, other = __rest(_a, ["children"]);
    return (_jsx("thead", Object.assign({ css: {
            display: "table-header-group"
        } }, other, { children: _jsx(TableSectionContext.Provider, Object.assign({ value: { type: "TableHead" } }, { children: children }), void 0) }), void 0));
}
export const TableRow = (_a) => {
    var { onClick, children } = _a, other = __rest(_a, ["onClick", "children"]);
    const theme = useTheme();
    const { type: tableSectionType } = React.useContext(TableSectionContext);
    const buttonProps = onClick
        ? {
            role: "button",
            tabIndex: 0
        }
        : {};
    return (_jsx("tr", Object.assign({ onClick: onClick, css: {
            height: tableSectionType === "TableHead" ? "31px" : "49px",
            display: "table-row",
            outline: "none",
            verticalAlign: "middle",
            cursor: onClick ? "pointer" : "default",
            ":hover": {
                background: onClick ? theme.colors.background.tint1 : "none"
            }
        } }, buttonProps, other, { children: children }), void 0));
};
TableRow.propTypes = {
    onClick: PropTypes.func,
    children: PropTypes.node
};
/**
 * Cell TableCell, used for both <td> and <th> elements.
 * textAlign: "justify" 导致字间距自动变大，感觉不规整，不如textAlign: "left"。
 */
const tableCellAlignments = {
    right: css({
        textAlign: "right",
        flexDirection: "row-reverse"
    }),
    left: css({
        textAlign: "left"
    }),
    center: css({
        textAlign: "center"
    }),
    justify: css({
        textAlign: "justify"
    })
};
/**
 * 旧版本的Cell组件，新版本改用Cell; 两个版本的样式不一样
 */
export const TableCell = (_a) => {
    var { align = "left", variant, component, ellipsis, children } = _a, other = __rest(_a, ["align", "variant", "component", "ellipsis", "children"]);
    const theme = useTheme();
    const { type: tableSectionType } = React.useContext(TableSectionContext);
    const Component = component || (tableSectionType === "TableHead" ? "th" : "td");
    const type = variant || (tableSectionType === "TableHead" ? "head" : "body");
    return (_jsx(Component, Object.assign({ css: [
            {
                zIndex: 4,
                position: "relative",
                borderBottom: "1px solid",
                borderColor: tableSectionType === "TableBody"
                    ? theme.colors.border.muted
                    : theme.colors.border.default,
                display: "table-cell",
                padding: `${theme.spaces.xs} ${theme.spaces.sm}`,
                [theme.mediaQueries.lg]: {
                    paddingLeft: theme.spaces.md,
                    paddingRight: theme.spaces.md
                },
                ":last-child": {
                    paddingRight: theme.spaces.md
                }
            },
            ellipsis && {
                whiteSpace: "nowrap",
                textOverflow: "ellipsis",
                overflow: "hidden"
            },
            type === "head"
                ? {
                    fontWeight: 500,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.muted
                }
                : {
                    fontWeight: 400,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.default
                },
            tableCellAlignments[align]
        ], scope: "col" }, other, { children: children }), void 0));
};
TableCell.propTypes = {
    align: PropTypes.oneOf(["right", "left", "center", "justify"]),
    variant: PropTypes.oneOf(["body", "head"]),
    // component: PropTypes.element,
    ellipsis: PropTypes.bool,
    children: PropTypes.node
};
/*可以通过interface来约定react组件props，还有必要用react的propTypes吗？
2个会报类型错误？TableCell.propTypes {align: PropTypes.oneOf( ； component: PropTypes.elementType,}
 这个地方似乎是打包层次用到的。  报错？只好删除了。
 可以只用typescript进行验证，也可以只用prop-types进行验证，也可以两者混搭进行验证，这个并没用严格限制。
首先不管用typescript，还是prop-types，都只在开发模式下进行检查。 PropTypes是组件接收prop的约束。
*/
/**
 * 新版 TableCell 组件，支持手机上作表格形式的报告，支持打印。
 * 做报表用的样式, CCell, RCell类似 Cell,就差align格式;
 * 旧版本请用TableCell组件,两个版本的样式不一样
 */
export const Cell = (_a) => {
    var { align = "left", variant, component, ellipsis, children } = _a, other = __rest(_a, ["align", "variant", "component", "ellipsis", "children"]);
    const theme = useTheme();
    const { type: tableSectionType } = React.useContext(TableSectionContext);
    const Component = component || (tableSectionType === "TableHead" ? "th" : "td");
    const type = variant || (tableSectionType === "TableHead" ? "head" : "body");
    return (_jsx(Component, Object.assign({ css: [
            {
                zIndex: 4,
                position: "relative",
                border: "1px solid",
                display: "table-cell",
                padding: `${theme.spaces.xs} ${theme.spaces.xs}`,
            },
            ellipsis && {
                whiteSpace: "nowrap",
                textOverflow: "ellipsis",
                overflow: "hidden"
            },
            type === "head"
                ? {
                    fontWeight: 500,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.muted
                }
                : {
                    fontWeight: 400,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.default
                },
            tableCellAlignments[align]
        ], scope: "col" }, other, { children: children }), void 0));
};
export function TableBody(_a) {
    var { children } = _a, other = __rest(_a, ["children"]);
    return (_jsx("tbody", Object.assign({ css: {
            display: "table-row-group"
        } }, other, { children: _jsx(TableSectionContext.Provider, Object.assign({ value: { type: "TableBody" } }, { children: children }), void 0) }), void 0));
}
export const ExpandingRow = ({ content, children }) => {
    const theme = useTheme();
    const [selected, setSelected] = React.useState(false);
    function close() {
        setSelected(false);
    }
    function open() {
        setSelected(true);
    }
    return (_jsxs(TableBody, { children: [_jsx(TableRow, Object.assign({ onClick: open }, { children: children }), void 0),
            selected && (_jsx("tr", Object.assign({ css: { display: "table-row", height: "100px" } }, { children: _jsxs("td", Object.assign({ colSpan: React.Children.count(children), css: {
                        borderBottom: 0,
                        width: "inherit",
                        padding: 0,
                        position: "relative"
                    } }, { children: [_jsx("div", Object.assign({ css: {
                                zIndex: 3,
                                width: "inherit",
                                position: "relative",
                                paddingBottom: "24px"
                            } }, { children: _jsx("div", Object.assign({ css: {
                                    position: "relative",
                                    whiteSpace: "normal",
                                    height: "auto",
                                    display: "block",
                                    paddingTop: "24px"
                                } }, { children: typeof content === "function" ? content(close) : content }), void 0) }), void 0),
                        _jsx(Layer, Object.assign({ css: {
                                position: "absolute",
                                top: "-49px",
                                left: "-16px",
                                right: "-16px",
                                borderRadius: theme.radii.md,
                                bottom: 0,
                                zIndex: 2
                            } }, { children: null }), void 0)] }), void 0) }), void 0))] }, void 0));
};
//被删除的部分 xxx.propTypes = {  };  是和rollup.js相关，一个模块打包工具/发布工具库，使用ES6的模块标准。
ExpandingRow.propTypes = {
    //content: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
    children: PropTypes.node
};
/**
大量重复的组件标签，简化: CCell代表居中显示的Cell
 */
export const CCell = (_a) => {
    var { align = "center", variant, component, ellipsis, children } = _a, other = __rest(_a, ["align", "variant", "component", "ellipsis", "children"]);
    return (Cell(Object.assign({ align,
        variant,
        component,
        ellipsis,
        children }, other)));
};
/**做点简化，免得看起来是一大堆重复性质代码：
 * 代表靠右边 显示的Cell
 */
export const RCell = (_a) => {
    var { align = "right", variant, component, ellipsis, children } = _a, other = __rest(_a, ["align", "variant", "component", "ellipsis", "children"]);
    return (Cell(Object.assign({ align,
        variant,
        component,
        ellipsis,
        children }, other)));
};
//# sourceMappingURL=Table.js.map